︠6c443d11-6d34-4a08-a56b-a96b4c214180i︠
%html
<h1 style="text-align: center;">Visualizing Numerical Integrals<br /></h1>
<p style="text-align: center;"><em>(c) 2009, written by Harald Schilly and <span>Minh Nguyen</span></em></p>
<p style="text-align: right;"><em><span>
<hr />
</span></em></p>
<h2 style="text-align: center;">Trapezoidal Rule<br /></h2>
<p style="text-align: left;">Approximating a definite integral $\int_a^b f(x) \;{\rm d}x$ for $x \in [a, b]$ using the <em>trapezoidal rule</em>. Here we assume that $a \leq b$. Let $n \in \mathbb{N}$ be a positive integer and set $\Delta_x = \frac{b-a}{n}$. Then we have the approximation</p>
<p style="text-align: left;">$$\int_a^b f(x) \; {\rm d}x \approx \sum_{i=0}^{n-1} \frac {f(x_i) + f(x_i + \Delta_x)} {2} \cdot \Delta_x$$</p>
<p style="text-align: left;">where $x_i = a + i \cdot \Delta_x$. When $i = 0$ we have $a = x_0$, and similarly $b = x_n$.</p>

︡50b91069-b0f8-4ec1-9f42-92ca3396bc18︡{"html": "<h1 style=\"text-align: center;\">Visualizing Numerical Integrals<br /></h1>\n<p style=\"text-align: center;\"><em>(c) 2009, written by Harald Schilly and <span>Minh Nguyen</span></em></p>\n<p style=\"text-align: right;\"><em><span>\n<hr />\n</span></em></p>\n<h2 style=\"text-align: center;\">Trapezoidal Rule<br /></h2>\n<p style=\"text-align: left;\">Approximating a definite integral $\\int_a^b f(x) \\;{\\rm d}x$ for $x \\in [a, b]$ using the <em>trapezoidal rule</em>. Here we assume that $a \\leq b$. Let $n \\in \\mathbb{N}$ be a positive integer and set $\\Delta_x = \\frac{b-a}{n}$. Then we have the approximation</p>\n<p style=\"text-align: left;\">$$\\int_a^b f(x) \\; {\\rm d}x \\approx \\sum_{i=0}^{n-1} \\frac {f(x_i) + f(x_i + \\Delta_x)} {2} \\cdot \\Delta_x$$</p>\n<p style=\"text-align: left;\">where $x_i = a + i \\cdot \\Delta_x$. When $i = 0$ we have $a = x_0$, and similarly $b = x_n$.</p>"}︡
︠957496de-c003-4518-ab35-5147168e2994a︠
%auto
# the function we are integrating numerically
var("x")
f(x) = 1/2 * x^2 - 2*x + sin(2*x)

def next_hue(dx):
    """
    Generator object for repetitive calls in the rgbcolor argument.
    """
    x = 0
    while True:
        if x > 1:
            x = 0
        yield hue(x, 0.3)
        x += dx
︡67765cb8-b1ad-4538-8e0e-e8add018f3b1︡︡
︠955140a5-f09c-4451-b2f9-30e0e6434298a︠
%auto
@interact
def _(ab=range_slider(-4, 10, 0.1, (-2,6), "[a, b]"), dx=slider(0.1,3,0.1,1,"dx")):
    a, b = ab
    xvals = srange(a, b, dx, include_endpoint=True)
    fx = [f(x) for x in xvals]
    fg = plot(f, (x,a,b))
    pts = point2d(zip(xvals,fx), rgbcolor="#ff3030", pointsize=30)
    lines = [([i,0], [i,f(i)], [i+dx, f(i+dx)], [i+dx, 0]) for i in xvals[0:-1]]
    nh = next_hue(1.0 / len(xvals))
    areas = sum(map(lambda p: polygon(p, rgbcolor=nh.next()), lines))
    show(fg + areas + pts)
    areas = [(f(i) + f(i+dx)) / 2 for i in xvals[0:-1]]
    print "approximation: %s" % float(dx * sum(areas))
    print "numerical integral: %s"  % numerical_integral(f, a, b)[0]
︡4ae5a026-d7e4-402c-88f9-13abcaac2699︡︡
︠2f3d6476-8828-4208-8aed-3e2bc3c759fbi︠
%html
<p>static example for fixed values, the above one is interactive</p>

︡d1350a3a-cf21-46d6-b4e3-544a89400b2c︡{"html": "<p>static example for fixed values, the above one is interactive</p>"}︡
︠745b8ac0-1c2d-4365-b2f4-67c8ce4a5d99ai︠
%hide
%auto
if True:
    a, b = -2, 6
    dx = 1
    xvals = srange(a, b, dx, include_endpoint=True)
    fx = [f(x) for x in xvals]
    fg = plot(f, (x,a,b))
    pts = point2d(zip(xvals,fx), rgbcolor="#ff3030", pointsize=30)
    lines = [([i,0], [i,f(i)], [i+dx, f(i+dx)], [i+dx, 0]) for i in xvals[0:-1]]
    nh = next_hue(1.0 / len(xvals))
    areas = sum(map(lambda p: polygon(p, rgbcolor=nh.next()), lines))
    show(fg + areas + pts)
    areas = [(f(i) + f(i+dx)) / 2 for i in xvals[0:-1]]
    print "approximation: %s" % float(dx * sum(areas))
    print "numerical integral: %s"  % numerical_integral(f, a, b)[0]
︡0c7f059c-c6c4-4c4a-b947-a438230b905d︡{"html": "<font color='black'><img src='cell://sage0.png'></font>"}︡{"stdout": "approximation: 5.51923393088\nnumerical integral: 4.58458454354"}︡
︠680e8014-fb53-41e7-97c2-45d42ea5ec18i︠
%html
<p style="text-align: center;">&nbsp;</p>
<hr />
<p>&nbsp;</p>
<h2 style="text-align: center;">Rectangle Rule</h2>
<p style="text-align: left;">Approximating a definite integral $\int_a^b f(x) \; {\rm d} x$ with $x \in [a,b]$ using rectangles. Again, we assume that $a \leq b$, $n \in \mathbb{N}$ is positive, and we set $\Delta_x = \frac {b - a} {n}$. Then we have the approximation</p>
<p style="text-align: left;">$$\int_a^b f(x) \; {\rm d} x \approx \frac{f(a) \cdot&nbsp; \Delta_x}{2}&nbsp; + \sum_{i=1}^{n-1} f(x_i) \cdot  \Delta_x&nbsp; + \frac{f(b) \cdot&nbsp; \Delta_x}{2}$$</p>
<p style="text-align: left;">where $x_i = a+ i \cdot \Delta_x$ for $1 \leq i \leq n-1$.</p>

︡31f260e9-cfb1-484e-970c-e6d94b7d4b39︡{"html": "<p style=\"text-align: center;\">&nbsp;</p>\n<hr />\n<p>&nbsp;</p>\n<h2 style=\"text-align: center;\">Rectangle Rule</h2>\n<p style=\"text-align: left;\">Approximating a definite integral $\\int_a^b f(x) \\; {\\rm d} x$ with $x \\in [a,b]$ using rectangles. Again, we assume that $a \\leq b$, $n \\in \\mathbb{N}$ is positive, and we set $\\Delta_x = \\frac {b - a} {n}$. Then we have the approximation</p>\n<p style=\"text-align: left;\">$$\\int_a^b f(x) \\; {\\rm d} x \\approx \\frac{f(a) \\cdot&nbsp; \\Delta_x}{2}&nbsp; + \\sum_{i=1}^{n-1} f(x_i) \\cdot  \\Delta_x&nbsp; + \\frac{f(b) \\cdot&nbsp; \\Delta_x}{2}$$</p>\n<p style=\"text-align: left;\">where $x_i = a+ i \\cdot \\Delta_x$ for $1 \\leq i \\leq n-1$.</p>"}︡
︠467db067-017b-4e53-b7ac-c317cc5d63e3a︠
%auto
@interact
def _(ab=range_slider(-4,10,0.1,(-2,6),"[a, b]"), dx=slider(0.1,3,0.1,0.5,"dx")):
    a, b = ab
    xvals = srange(a, b, dx, include_endpoint=True)
    fx = [f(x) for x in xvals]
    fg = plot(f, (x,a,b))
    pts = point2d(zip(xvals,fx), rgbcolor="#ff3030", pointsize=30)
    #lines = [([i,0], [i,f(i)], [i+dx, f(i+dx)], [i+dx, 0]) for i in xvals[0:-1]]
    lines = [([xvals[0], 0], [xvals[0], f(xvals[0])], [xvals[0] + dx/2, f(xvals[0])], [xvals[0] + dx/2, 0])]
    for i in xvals[1:-1]:
        x1, x2 = i-dx/2, i+dx/2
        lines.append(([x1,0], [x1, f(i)], [x2, f(i)], [x2, 0]))
    x1, x2 = xvals[-1]-dx/2, xvals[-1]
    lines.append(([x1,0], [x1, f(x2)], [x2, f(x2)], [x2, 0]))
    nh = next_hue(1.0 / len(xvals))
    areas = sum(map(lambda p: polygon(p, rgbcolor=nh.next()), lines))
    show(fg + areas + pts)
    areas = [f(i) for i in xvals[1:-1]]
    areas.append(f(xvals[0]) / 2)
    areas.append(f(xvals[-1]) / 2)
    print "approximation: %s " % float(dx * sum(areas))
    print "numerical integral: %s"  % numerical_integral(f, a, b)[0]
︡8bceb2b6-bbed-442a-b4fb-7ee27bde4e2f︡︡
︠5897fbd4-165f-4669-93da-c895a19235ffi︠
%html
<p>static example with fixed values, the above one is interactive</p>

︡cc649227-50d5-4777-bb58-d8b4c6ab2bc5︡{"html": "<p>static example with fixed values, the above one is interactive</p>"}︡
︠95ee08e3-4817-4139-8de0-5facf401a550ai︠
%hide
%auto
if True:
    a, b = -2, 6
    dx = 0.5
    xvals = srange(a, b, dx, include_endpoint=True)
    fx = [f(x) for x in xvals]
    fg = plot(f, (x,a,b))
    pts = point2d(zip(xvals,fx), rgbcolor="#ff3030", pointsize=30)
    #lines = [([i,0], [i,f(i)], [i+dx, f(i+dx)], [i+dx, 0]) for i in xvals[0:-1]]
    lines = [([xvals[0], 0], [xvals[0], f(xvals[0])], [xvals[0] + dx/2, f(xvals[0])], [xvals[0] + dx/2, 0])]
    for i in xvals[1:-1]:
        x1, x2 = i-dx/2, i+dx/2
        lines.append(([x1,0], [x1, f(i)], [x2, f(i)], [x2, 0]))
    x1, x2 = xvals[-1]-dx/2, xvals[-1]
    lines.append(([x1,0], [x1, f(x2)], [x2, f(x2)], [x2, 0]))
    nh = next_hue(1.0 / len(xvals))
    areas = sum(map(lambda p: polygon(p, rgbcolor=nh.next()), lines))
    show(fg + areas + pts)
    areas = [f(i) for i in xvals[1:-1]]
    areas.append(f(xvals[0]) / 2)
    areas.append(f(xvals[-1]) / 2)
    print "approximation: %s " % float(dx * sum(areas))
    print "numerical integral: %s"  % numerical_integral(f, a, b)[0]
︡c743469d-fc56-4c6f-86ce-04d8d9838a2f︡{"html": "<font color='black'><img src='cell://sage0.png'></font>"}︡{"stdout": "approximation: 4.81471226681 \nnumerical integral: 4.58458454354"}︡
